type var = Literal of int | Id of string

type stitch = Knit of var | Purl of var | Increase | Decrease

type repeat = Times of var | Until of var

type row = {
    row_name : string;
    row_formals : string list;
    row_start : stitch list;
    row_repeat : stitch list;
    repeat_condition : repeat;
    row_end : stitch list;
  }   

type call = {
    called_name : string;
    actuals : var list;
  }

type stitch_pattern = {
    stitch_pattern_name : string;
    stitch_pattern_formals : string list;
    rows : call list;
  }

type pattern = {
    cast_on : int;
    work : (call * repeat) list;
  }

type program = row list * stitch_pattern list * pattern option

let string_of_var = function
    Literal(l) -> string_of_int l
  | Id(name)   -> name

let string_of_stitch = function
    Knit(count) -> "k" ^ string_of_var count
  | Purl(count) -> "p" ^ string_of_var count
  | Increase    -> "inc"
  | Decrease    -> "dec"

let string_of_repeat = function
    Times(count)  -> string_of_var count ^ " times"
  | Until(remain) -> "until " ^ string_of_var remain ^ " stitches remain"

let string_of_row row =
  row.row_name ^ " Row (" ^ String.concat ", " row.row_formals ^ "): " ^
  String.concat "," (List.map string_of_stitch row.row_start) ^
  " * " ^  String.concat "," (List.map string_of_stitch row.row_repeat) ^
  " * repeat from * to * " ^ string_of_repeat row.repeat_condition ^
  " " ^ String.concat "," (List.map string_of_stitch row.row_end) ^ "\n"

let string_of_call call = call.called_name ^ "(" ^
  String.concat ", " (List.map string_of_var call.actuals) ^ ")"

let string_of_stitch_pattern stitch_pattern = 
  stitch_pattern.stitch_pattern_name ^ " Stitch Pattern (" ^
  String.concat ", " stitch_pattern.stitch_pattern_formals ^ "):\n" ^
  String.concat "\n" (List.map string_of_call stitch_pattern.rows) ^ "\n"

let string_of_work (call, repeat) =
  "Work " ^ string_of_call call ^ " " ^ string_of_repeat repeat

let string_of_pattern = function
    None -> "WHOOPS!!! No pattern?!?!?!"
  | Some(pattern) -> "CO " ^ string_of_int pattern.cast_on ^ "\n" ^
    String.concat "\n" (List.map string_of_work pattern.work) ^
    "\nBO"

let string_of_program (rows, stitch_patterns, pattern) =
  String.concat "\n" (List.map string_of_row rows) ^ "\n" ^
  String.concat "\n" (List.map string_of_stitch_pattern stitch_patterns) ^
  "\n" ^ string_of_pattern pattern ^ "\n"

