open Ast
open Translate

(** Translate a program in AST form into the text of a C program. *)
let translate (rows, stitch_patterns, pattern) =
  let translate_stitch = function
      Knit(var) -> for_loop (string_of_var var) "make_stitch(KNIT);"
    | Purl(var) -> for_loop (string_of_var var) "make_stitch(PURL);"
    | Increase ->
        "make_stitch(INCREASE);prev_row=prev_row->next;make_stitch(INCREASE);"
    | Decrease ->
        "make_stitch(DECREASE);prev_row->next_row=curr;prev_row=prev_row->prev;"
  in
  let translate_repeat body = function
      Times(var) -> "for(repeat=0;repeat<" ^ (string_of_var var) ^ ";repeat++)"
      ^ "{\n" ^ body ^ "\n}"
    | Until(var) -> "do {" ^ body ^ "}"
      ^ "while(prev_row!=NULL && prev_row->count!=" ^ string_of_var var ^ ");"
  in
  let translate_row row =
    "\nvoid " ^ row.row_name ^ "("
    ^ (String.concat "," (List.map (fun x -> ("int " ^ x) ) row.row_formals))
    ^ ") {"
    ^ debug ("printf(\"" ^ row.row_name ^ "\\n\");\n") ^ "\n"
    ^ (String.concat "\n" (List.map translate_stitch row.row_start)) ^ "\n"
    ^ translate_repeat
      (String.concat "\n" (List.map translate_stitch row.row_repeat)) 
      row.repeat_condition
    ^ "\n" ^ (String.concat "\n" (List.map translate_stitch row.row_end))
    ^ "\nif (prev_row != NULL) {\n"
    ^ "  printf(\"ERROR! %d stitches remain after working %s Row.\\n\","
    ^ " prev_row->count, \"" ^ row.row_name ^ "\");\n"
    ^ "  exit(0);\n"
    ^ "}\n}"
  in
  let translate_call call =
    call.called_name ^ "("
    ^ (String.concat "," (List.map string_of_var call.actuals)) ^ ");" in
  let translate_stitch_pattern stitch_pattern =
    "\nvoid " ^ stitch_pattern.stitch_pattern_name ^ "("
    ^ String.concat ", " stitch_pattern.stitch_pattern_formals ^ ") {\n"
    ^ debug ("printf(\"" ^ stitch_pattern.stitch_pattern_name ^ "\\n\");\n")
    ^ String.concat new_row (List.map translate_call stitch_pattern.rows)
    ^ "\n}\n\n" in
  let translate_pattern_repeat (call, repeat) =
    translate_repeat (new_row ^ translate_call call ^ "prev_row=curr;") repeat
  in
  match pattern with
    None -> ":("
  | Some(pattern) ->
    preamble
    ^ String.concat "\n" (List.map translate_row rows)
    ^ String.concat "\n" (List.map translate_stitch_pattern stitch_patterns)
    ^ "\nint main(int argc, char* argv[]) {\n"
    ^ "int repeat;\n"
    ^ "struct stitch* first_stitch;\n"
    ^ cast_on pattern.cast_on
    ^ "first_stitch=curr;\n"
    ^ String.concat "\n" (List.map translate_pattern_repeat pattern.work)
    ^ final_output

