%{ open Ast %}

%token LPAREN RPAREN COMMA COLON STAR
%token KNIT PURL INC DEC CO BO
%token WORK REPEAT TIMES UNTIL REMAIN
%token ROW STPATT
%token <int> LITERAL
%token <string> ID
%token EOL EOF

%start program
%type <Ast.program> program

%%

program:
   /* nothing */ { [], [], None }
 | program EOL { $1 }
 | program row { let (rows, stitch_patterns, pattern) = $1 in
                $2 :: rows, stitch_patterns, pattern }
 | program stitch_pattern { let (rows, stitch_patterns, pattern) = $1 in
                           rows, $2 :: stitch_patterns, pattern }
 | program pattern { let (rows, stitch_patterns, _ )  = $1 in
                           rows, stitch_patterns, $2 }
 | program error { print_endline "KYRA :("; ([], [], None) }

row:
  ID ROW LPAREN formals_opt RPAREN COLON
    stitch_list_opt STAR stitch_list STAR REPEAT repeat stitch_list_opt EOL
    { { row_name = $1;
        row_formals = $4;
        row_start = $7;
        row_repeat = List.rev $9;
        repeat_condition = $12;
        row_end = $13 } }

formals_opt:
    /* nothing */ { [] }
  | formal_list   { List.rev $1 }

formal_list:
    ID                   { [$1] }
  | formal_list COMMA ID { $3 :: $1 }

stitch_pattern:
  ID STPATT LPAREN formals_opt RPAREN COLON EOL call_list EOL { {
    stitch_pattern_name = $1;
    stitch_pattern_formals = $4;
    rows = List.rev $8; } }

pattern:
  CO LITERAL EOL work_list BO EOL { Some({ cast_on = $2; work = List.rev $4; }) }

call:
  ID LPAREN vars_opt RPAREN { { called_name = $1; actuals = $3; } }

call_list:
    /* nothing */  { [] }
  | call_list call EOL { $2 :: $1 }

work_list:
    /* nothing */         { [] }
  | work_list WORK call repeat EOL { ($3, $4) :: $1 }

repeat:
    var TIMES        { Times($1) }
  | UNTIL var REMAIN { Until($2) }

stitch_list_opt:
    /* nothing */ { [] }
  | stitch_list   { List.rev $1 }

stitch_list:
    stitch { [$1] }
  | stitch_list COMMA stitch { $3 :: $1 }

stitch:
    KNIT var { Knit($2) }
  | PURL var { Purl($2) }
  | INC      { Increase }
  | DEC      { Decrease }

vars_opt:
    /* nothing */ { [] }
  | var_list      { List.rev $1 }

var_list:
    var                { [$1] }
  | var_list COMMA var { $3 :: $1 }

var:
    LITERAL { Literal($1) }
  | ID      { Id($1) }

