(* To turn on debug output in the C program generated by the compiler,
  change this debug function to return the log statement instead of "" *)
let debug log = ""
(* Code snippets to be included in the generated C program. *)
let includes =
    "#include <stdio.h>\n" ^
    "#include <stdlib.h>\n\n" 
let stitch_enum =
    "enum stitch_type { KNIT, PURL, INCREASE, DECREASE, CO };\n\n"
let stitch_struct =
    "struct stitch {\n" ^
    "  int right_side;\n" ^
    "  int count;\n" ^
    "  enum stitch_type type;\n" ^
    "  struct stitch* next;\n" ^
    "  struct stitch* prev;\n" ^
    "  struct stitch* next_row;\n" ^
    "  struct stitch* prev_row;\n" ^
    "};\n\n"
let globals =
    "struct stitch* curr;\nstruct stitch* prev_row;\n" ^
    "int i, repeat;\n"
let print_stitch_symbol =
    "void print_symbol(enum stitch_type type, int rs) {\n" ^
    "  char symbol = 'x';\n" ^
    "  if ((type == KNIT && rs) || (type == PURL && !rs)) { symbol = '.'; }\n" ^
    "  if ((type == PURL && rs) || (type == KNIT && !rs)) { symbol = '-'; }\n" ^
    "  if (type == INCREASE) { symbol = 'v'; }\n" ^
    "  if (type == DECREASE) { symbol = '/'; }\n" ^
    "  if (type == CO) { symbol = '_'; }\n" ^
    "  printf(\"%c \", symbol);\n" ^
    "}"
let make_stitch =
    "void make_stitch(enum stitch_type type) {\n" ^
    debug "  printf(\"type %d\t\", type);\n" ^
    "  struct stitch* new_stitch =\n" ^
    "      (struct stitch*) malloc(sizeof(struct stitch));\n" ^
    "  new_stitch->type = type;\n" ^
    "  new_stitch->next_row = NULL;\n" ^
    "  new_stitch->next = NULL;\n" ^
    "  new_stitch->prev = curr;\n" ^
    "  int count = 1;\n" ^
    "  if (curr != NULL) {\n" ^
    "    curr->next = new_stitch;\n" ^
    "    count = curr->count + 1;\n" ^
    "  }\n" ^
    "  new_stitch->count = count;\n" ^
    debug "  printf(\"stitch count: %d\\t\", count);\n" ^
    "  new_stitch->prev_row = prev_row;\n" ^
    "  if (prev_row != NULL) {\n" ^
    debug "    printf(\"prev row count: %d\\n\", prev_row->count);\n" ^
    "    prev_row->next_row = new_stitch;\n" ^
    "    prev_row = prev_row->prev;\n" ^
    "  } else if (type != CO) {" ^
    "    printf(\"ERROR! No stitch available to work type %d\\n\", type);" ^
    "    exit(0);\n" ^
    "  }" ^
    "  curr = new_stitch;\n" ^
    "}\n"
(* Functions and definitions to be included at the beginning of the generated
  program text. *)
let preamble = includes ^ stitch_enum ^ stitch_struct ^ globals
    ^ print_stitch_symbol ^ make_stitch
let for_loop times body = "for (i=0;i<" ^ times ^ ";i++){" ^ body ^ "}"
let new_row = "\nprev_row=curr;curr=NULL;\n"
let cast_on count =
    "make_stitch(CO);\n" ^
    "int i;\n" ^
    "for (i = 1; i < " ^ string_of_int count ^ "; i++) {\n" ^
    "make_stitch(CO);\n" ^
    "}\n"
(* Final program output options. *)
let print_row_count =
  debug "printf(\"row_count\\n\");"
  ^ "curr=first_stitch;i=0;\nwhile(curr!=NULL){i++; curr=curr->next_row;}"
  ^ "\nprintf(\"This pattern has %d rows.\\n\",i);\n"
let print_stitch_count =
  debug "printf(\"stitch_count\\n\");" ^ "curr=first_stitch;\n"
  ^ "if(argc > 2){\n"
  ^ "  repeat=atoi(argv[2]);i=1;\n"
  ^ "  while(curr->next_row!=NULL && i < repeat){i++;curr=curr->next_row;}}\n"
  ^ "  repeat=i;i=0;\n"
  ^ "  if(curr->next!=NULL){while(curr!=NULL){i++;curr=curr->next;}}\n"
  ^ "  else{i=curr->count;}\n"
  ^ "  printf(\"Row %d of this pattern has %d stitches.\\n\",repeat,i);\n"
let print_chart =
  debug "printf(\"print_chart\\n\");"
  ^ "curr=first_stitch;\n"
  ^ "while(curr->next_row!=NULL){curr=curr->next_row;}\n"
  ^ "first_stitch=curr;\n"
  ^ "while(first_stitch!=NULL){\n"
  ^ "  if(curr->next!=NULL){\n"
  ^ "    while(curr!=NULL){print_symbol(curr->type,1);curr=curr->next;}\n"
  ^ "  } else {\n"
  ^ "    while(curr!=NULL){print_symbol(curr->type,0);curr=curr->prev;}\n  }\n"
  ^ "  printf(\"\\n\");first_stitch=first_stitch->prev_row;curr=first_stitch;"
  ^ "\n}\n"
let final_output =
  "if(first_stitch->next_row!=NULL){first_stitch=first_stitch->next_row;}\n"
  ^ "if (argc > 1) {\n"
  ^ "if (!strcmp(argv[1], \"--row_count\")) {" ^ print_row_count ^ "}\n"
  ^ "if (!strcmp(argv[1], \"--stitch_count\")) {" ^ print_stitch_count ^ "}\n"
  ^ "if (!strcmp(argv[1], \"--print_chart\")) {" ^ print_chart ^ "}\n"
  ^ "}\n}\n"
